@@ -3,8 +3,9 @@ from __future__ import annotations
 from typing import Union, Dict, List, Any, Tuple
 from asyncio import gather
 
-from .protocol import BaseProtocol, support_protocol
-from .exception import ProtocolUnsopportError
+from .protocol import BaseProtocol, SupportProtocol
+from .exception import ProtocolUnsopportError, NameConflictError
+from .logger import logger
 
 
 class ServiceStatus:
@@ -40,20 +41,29 @@ class ServiceStatus:
                 return
         raise KeyError(key)
 
-    def register_service(self, protocol: str, *args, **kw):
-        if protocol not in support_protocol():
+    def __len__(self) -> int:
+        return len(self.__bind_services)
+
+    def register_service(self, protocol: str, *args, **kw) -> BaseProtocol:
+        if protocol not in SupportProtocol.get():
             raise ProtocolUnsopportError
-        self.bind_service(BaseProtocol._support_protocol[protocol](*args, **kw))
+        target_instance = SupportProtocol.SUPPORT_PROTOCOL[protocol](*args, **kw)
+        self.bind_service(target_instance)
+        return target_instance
 
-    def bind_service(self, service: BaseProtocol) -> None:
+    def bind_service(self, service: BaseProtocol) -> BaseProtocol:
+        if service in self:
+            raise NameConflictError
         self.__bind_services.append(service)
+        return service
 
-    def unbind_service(self, unbind_service: Union[BaseProtocol, str]) -> None:
-        if isinstance(unbind_service, str):
-            unbind_service = self[unbind_service]
+    def unbind_service(self, unbind_service: Union[BaseProtocol, str]) -> BaseProtocol:
         if unbind_service not in self:
             raise KeyError
+        if isinstance(unbind_service, str):
+            unbind_service = self[unbind_service]
         self.__bind_services.pop(self.__bind_services.index(unbind_service))
+        return unbind_service
 
     async def get_detect_result(self) -> Dict[str, bool]:
         tasks = [service.detect() for service in self]
@@ -67,18 +77,19 @@ class ServiceStatus:
     def load(cls, source: Dict[str, List]) -> ServiceStatus:
         instance = cls()
         for key, value in source.items():
-            if key not in support_protocol():
-                raise ProtocolUnsopportError(f"Unsopported protocol: {key} !")
+            # logger.debug(f"Loading protocol {key} with {value}")
+            if key not in SupportProtocol.get():
+                logger.error(f"Unsopported protocol: {key} !")
+                logger.warning(f"Protocol {key} will ignored from loading")
+                continue
             for this_service_config in value:
-                this_service_instance = BaseProtocol._support_protocol[key].load(
-                    this_service_config
-                )
+                this_service_instance = SupportProtocol.SUPPORT_PROTOCOL[key].load(this_service_config)
                 instance.bind_service(this_service_instance)
         return instance
 
     def export(self) -> Dict[str, List[Dict]]:
         ret_dict = {}
-        for i in support_protocol():
+        for i in SupportProtocol.get():
             ret_dict[i] = []
             for j in self:
                 if j._PROTOCOL_NAME != i:
@@ -122,6 +133,9 @@ class ServiceStatusGroup:
                 return
         raise KeyError(key)
 
+    def __len__(self):
+        return len(self.__bind_services_group)
+
     def items(self):
         return self.__bind_services_group.items()
 
@@ -134,10 +148,12 @@ class ServiceStatusGroup:
         for this_service in services:
             self.__bind_services_group[name].bind_service(this_service)
 
-    def unbind_group(self, key: str) -> None:
+    def unbind_group(self, key: str) -> ServiceStatus:
         if key not in self:
             raise KeyError
+        temp_ret = self.__bind_services_group[key]
         del self.__bind_services_group[key]
+        return temp_ret
 
     async def get_detect_result(self) -> Dict[str, bool]:
         ret_result = {}
@@ -153,9 +169,7 @@ class ServiceStatusGroup:
     def load(cls, source: Dict[str, Dict[str, List]]) -> ServiceStatusGroup:
         instance = cls()
         for name, service_status_config in source.items():
-            instance.__bind_services_group[name] = ServiceStatus.load(
-                service_status_config
-            )
+            instance.__bind_services_group[name] = ServiceStatus.load(service_status_config)
         return instance
 
     def export(self) -> Dict[str, Dict[str, List]]:
